package org.jeecg.common.exception;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.AuthorizationException;
import org.apache.shiro.authz.UnauthorizedException;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.enums.SentinelErrorInfoEnum;
import org.springframework.core.NestedRuntimeException;
import org.springframework.dao.*;
import org.springframework.data.redis.connection.PoolException;
import org.springframework.http.HttpStatus;
import org.springframework.jdbc.BadSqlGrammarException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.NoHandlerFoundException;

import javax.servlet.http.HttpServletRequest;
import java.sql.SQLException;
import java.sql.SQLNonTransientException;
import java.sql.SQLSyntaxErrorException;
import java.util.Iterator;

/**
 * 异常处理器
 */
@RestControllerAdvice
@Slf4j
public class JeecgBootExceptionHandler {

	/**
	 * 处理自定义异常
	 */
	@ExceptionHandler(JeecgBootException.class)
	public Result<?> handleJeecgBootException(JeecgBootException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error(e.getMessage());
	}

	/**
	 * 处理自定义微服务异常
	 */
	@ExceptionHandler(JeecgCloudException.class)
	public Result<?> handleJeecgCloudException(JeecgCloudException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error(e.getMessage());
	}

	/**
	 * 处理自定义401异常
	 */
	@ExceptionHandler(JeecgBoot401Exception.class)
	@ResponseStatus(HttpStatus.UNAUTHORIZED)
	public Result<?> handleJeecgBoot401Exception(JeecgBoot401Exception e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return new Result<>(401, e.getMessage());
	}

	@ExceptionHandler(NoHandlerFoundException.class)
	public Result<?> handlerNoFoundException(Exception e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error(404, "路径不存在，请检查路径是否正确");
	}

	@ExceptionHandler(DuplicateKeyException.class)
	public Result<?> handleDuplicateKeyException(DuplicateKeyException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error("数据库中已存在该记录");
	}

	@ExceptionHandler({UnauthorizedException.class, AuthorizationException.class})
	public Result<?> handleAuthorizationException(AuthorizationException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.noauth("没有权限，请联系管理员授权");
	}

	@ExceptionHandler(HttpRequestMethodNotSupportedException.class)
	public Result<?> httpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e, HttpServletRequest request) {
		StringBuffer sb = new StringBuffer();
		sb.append("不支持");
		sb.append(e.getMethod());
		sb.append("请求方法，");
		sb.append("支持以下");

		String[] methods = e.getSupportedMethods();
		if (methods != null) {
			for (String str : methods) {
				sb.append(str);
				sb.append("、");
			}
		}

		log.error("Intercepted unsupported HTTP method request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), e.getMethod(), request.getParameterMap());

		log.error(sb.toString(), e);

		return Result.error(405, sb.toString());
	}

	@ExceptionHandler(MaxUploadSizeExceededException.class)
	public Result<?> handleMaxUploadSizeExceededException(MaxUploadSizeExceededException e, HttpServletRequest request) {
		if (request instanceof MultipartHttpServletRequest) {
			MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
			Iterator<String> fileNames = multipartRequest.getFileNames();

			while (fileNames.hasNext()) {
				String fileName = fileNames.next();
				MultipartFile file = multipartRequest.getFile(fileName);
				log.error("File upload exceeded max size: File name={}, Size={} bytes, Content type={}",
						file.getOriginalFilename(), file.getSize(), file.getContentType());
			}
		}

		return Result.error("文件大小超出10MB限制, 请压缩或降低文件质量!");
	}

	@ExceptionHandler(DataIntegrityViolationException.class)
	public Result<?> handleDataIntegrityViolationException(DataIntegrityViolationException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		//【issues/3624】数据库执行异常handleDataIntegrityViolationException提示有误 #3624
		return Result.error("执行数据库异常,违反了完整性例如：违反惟一约束、违反非空限制、字段内容超出长度等");
	}

	@ExceptionHandler(PoolException.class)
	public Result<?> handlePoolException(PoolException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error("Redis 连接异常!");
	}

	@ExceptionHandler({SQLSyntaxErrorException.class, SQLNonTransientException.class, SQLException.class})
	public Result<?> handleSQLSyntaxErrorException(SQLException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error("SQL语法错误！");
	}

	@ExceptionHandler({BadSqlGrammarException.class, InvalidDataAccessResourceUsageException.class, NonTransientDataAccessException.class, DataAccessException.class})
	public Result<?> handleBadSqlGrammarException(DataAccessException e, HttpServletRequest request) {
		log.error("Intercepted exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);
		return Result.error("SQL语法错误！");
	}

	@ExceptionHandler(Exception.class)
	public Result<?> handleException(Exception e, HttpServletRequest request) {
		log.error("Intercepted an exception for request: URL={}, Method={}, Parameters={}",
				request.getRequestURI(), request.getMethod(), request.getParameterMap(), e);

		// 处理Sentinel限流自定义异常
		Throwable throwable = e.getCause();
		SentinelErrorInfoEnum errorInfoEnum = SentinelErrorInfoEnum.getErrorByException(throwable);
		if (ObjectUtil.isNotEmpty(errorInfoEnum)) {
			return Result.error(errorInfoEnum.getError());
		}

		return Result.error("操作失败");
	}
}
