package org.jeecg.modules.tools;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ReflectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.jeecg.common.api.CommonAPI;
import org.jeecg.common.aspect.DictAspect;
import org.jeecg.common.aspect.annotation.Dict;
import org.jeecg.common.util.oConvertUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;

/**
 * 字典翻译辅助工具类
 */
@Component
public class DictUtils {

	private static final Logger log = LoggerFactory.getLogger(DictAspect.class);

	private CommonAPI commonAPI;

	@Autowired
	public void setCommonAPI(CommonAPI commonAPI) {
		this.commonAPI = commonAPI;
	}

	/**
	 * 翻译字典，支持@Dict的翻译，翻译出的字段放在有@Dict字段的名字 + "_dictText"
	 * @param o 传入的是单个entity对象
	 * @return 返回的是json对象
	 */
	public JSONObject translateDict(Object o){
		Field[] fields = ReflectUtil.getFields(o.getClass());
		ObjectMapper mapper = new ObjectMapper();
		String json = "{}";
		try {
			json = mapper.writeValueAsString(o);
		} catch (JsonProcessingException e) {
			log.error("json解析失败" + e.getMessage(), e);
		}
		JSONObject item = JSONObject.parseObject(json);
		for (Field field:fields){
			if (field.getAnnotation(Dict.class) != null) {
				String key = String.valueOf(item.get(field.getName()));
				String textValue = getDictTxt(field,key);
				item.put(field.getName() + "_dictText", textValue);
			}
		}
		return item;
	}

	/**
	 * 翻译字典，支持@Dict的翻译，翻译出的字段放在有@Dict字段的名字 + "_dictText"<br/>
	 * 在clazz与o不是一种对象时使用<br/>
	 * 如果需要保持一致请使用{@link org.jeecg.modules.tools.DictUtils#translateDictSelf}
	 * @param o 传入的是单个entity对象
	 * @param clazz 返回对象的类型
	 * @return 传入的clazz类型的新对象
	 * @throws IllegalAccessException
	 */
	public <T> T translateDict(Object o,Class<T> clazz) throws IllegalAccessException {
		Field[] fields = ReflectUtil.getFields(clazz);
		T rObject = ReflectUtil.newInstance(clazz);
		//将所有内容转入新对象里
		BeanUtil.copyProperties(o,rObject);
		return translateDictSelf(rObject,clazz);
	}

	/**
	 * 翻译字典，支持@Dict的翻译，翻译出的字段放在有@Dict字段的名字 + "_dictText"<br/>
	 * 此方法返回自己，保证是同一个对象，以方便list使用<br/>
	 * 如果需要保持一致请使用{@link org.jeecg.modules.tools.DictUtils#translateDict}
	 * @param o 传入的是单个entity对象,也是返回的对象
	 * @param clazz 返回对象的类型
	 * @return 返回的是变量o这个对象
	 * @throws IllegalAccessException
	 */
	public <T> T translateDictSelf(T o,Class<T> clazz) throws IllegalAccessException {
		Field[] fields = ReflectUtil.getFields(clazz);
		for (Field field:fields){
			if (field.getAnnotation(Dict.class) != null) {
				boolean accessible = field.isAccessible();
				field.setAccessible(true);
				String key = String.valueOf(field.get(o));
				field.setAccessible(accessible);
				String textValue = getDictTxt(field,key);
				Field newField = ReflectUtil.getField(clazz,field.getName() + "_dictText");
				//为空则赋值失败
				if(newField != null){
					boolean accessibleNew = newField.isAccessible();
					newField.setAccessible(true);
					newField.set(o,textValue);
					newField.setAccessible(accessibleNew);
				}else{
					log.error("字段【{}{}】不存在！值为：{} ",field.getName(),"_dictText",textValue);
				}
			}
		}
		return o;
	}

	/**
	 * 获取根据属性和真实的值获取字典值
	 * @param field 反射出来的类属性
	 * @param key 查询的值
	 * @return 字典文本
	 */
	private String getDictTxt(Field field,String key){
		boolean accessible = field.isAccessible();
		field.setAccessible(true);
		String code = field.getAnnotation(Dict.class).dicCode();
		String text = field.getAnnotation(Dict.class).dicText();
		String table = field.getAnnotation(Dict.class).dictTable();
		String textValue = translateDictValue(code, text, table, key);
		log.debug(" 字典Val : " + textValue);
		log.debug(" __翻译字典字段__ " + field.getName() + "_dictText" + "： " + textValue);
		field.setAccessible(accessible);
		return textValue;
	}

	/**
	 * 翻译字典
	 * @param code 字典code字段
	 * @param key 字典查询的值
	 * @return 字典文本
	 */
	public String translateDictValue(String code, String key) {
		return translateDictValue(code,"","",key);
	}

	/**
	 * 翻译字典
	 * @param code 字典code字段
	 * @param text 字典文字字段
	 * @param table 字典表名
	 * @param key 字典查询的值
	 * @return 字典文本
	 */
	public String translateDictValue(String code, String text, String table, String key) {
		if (oConvertUtils.isEmpty(key)) {
			return null;
		} else {
			StringBuilder textValue = new StringBuilder();
			String[] keys = key.split(",");
			for(int i = 0; i < keys.length; ++i) {
				String k = keys[i];
				String tmpValue;
				log.debug(" 字典 key : " + k);
				if (k.trim().length() != 0) {
					if (!StringUtils.isEmpty(table)) {
						log.debug("--DictUtils------dicTable=" + table + " ,dicText= " + text + " ,dicCode=" + code);
						tmpValue = this.commonAPI.translateDictFromTable(table, text, code, k.trim());
					} else {
						tmpValue = this.commonAPI.translateDict(code, k.trim());
					}
					if (tmpValue != null) {
						if (!"".equals(textValue.toString())) {
							textValue.append(",");
						}
						textValue.append(tmpValue);
					}
				}
			}
			return textValue.toString();
		}
	}
}
