package org.jeecg.modules.tools.office.convert;

import org.apache.poi.poifs.filesystem.FileMagic;
import org.apache.poi.ss.usermodel.PrintSetup;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

public final class MyOfficeUtils {

	private MyOfficeUtils () {}

	/**
	 * 获取文档类型
	 * @param file 待转换的文件
	 * @return 文件转换器
	 * @throws IOException io异常
	 */
	public static FileMagic getFileMagic(File file) throws IOException {
		try (FileInputStream inp = new FileInputStream(file)) {
			return getFileMagic(inp);
		}
	}

	/**
	 * 获取文档类型
	 * @param inp 待转换的文件流
	 * @return 文件转换器
	 * @throws IOException io异常
	 */
	public static FileMagic getFileMagic(InputStream inp) throws IOException {
		InputStream is = FileMagic.prepareToCheckMagic(inp);
		return FileMagic.valueOf(is);
	}

	/**
	 * 判断是否excel03版本
	 * @param file 待检测的文件
	 * @return true：是03版本 false：不是03版本
	 */
	public static boolean isExcel03(File file) {
		try {
			boolean ole2 = FileMagic.OLE2 == getFileMagic(file);
			if (!ole2) {
				return false;
			}
			try (Workbook workbook = WorkbookFactory.create(file)) {
				if(workbook != null) return true;
			}
			return false;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * 判断是否excel07版本
	 * @param file 待检测的文件
	 * @return true：是07版本 false：不是07版本
	 */
	public static boolean isExcel07(File file) {
		try {
			boolean ole2 = FileMagic.OOXML == getFileMagic(file);
			if (!ole2) {
				return false;
			}
			try (Workbook workbook = WorkbookFactory.create(file)) {
				if(workbook != null) return true;
			}
			return false;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * excel默认打印配置
	 * @param workbook 文档对象
	 */
	public static void setDefaultExcelPrintScale (Workbook workbook) {
		int numberOfSheets = workbook.getNumberOfSheets();
		for (int i = 0; i < numberOfSheets; i++) {
			workbook.removePrintArea(i);
			setSheetPrintScale(workbook.getSheetAt(i), null);
		}
	}

	static void setExcelPrintScale (Workbook workbook, OfficeInputExcelPrintSetup inputExcelPrintSetup) {
		int numberOfSheets = workbook.getNumberOfSheets();
		for (int i = 0; i < numberOfSheets; i++) {
			workbook.removePrintArea(i);
			setSheetPrintScale(workbook.getSheetAt(i), inputExcelPrintSetup);
		}
	}

	private static void setSheetPrintScale(Sheet sheet, OfficeInputExcelPrintSetup inputExcelPrintSetup) {
		PrintSetup setup = sheet.getPrintSetup();
		if (inputExcelPrintSetup == null) {
			setup.setLandscape(true);
			setup.setFitHeight((short) 0);
			setup.setFitWidth((short) 1);
			setup.setPaperSize(PrintSetup.A4_PAPERSIZE);
			sheet.setFitToPage(true);
			sheet.setPrintRowAndColumnHeadings(false);
			sheet.setHorizontallyCenter(true);
			sheet.setVerticallyCenter(false);
		} else {
			setup.setLandscape(inputExcelPrintSetup.isLandscape());
			setup.setFitHeight(inputExcelPrintSetup.getFitHeight());
			setup.setFitWidth(inputExcelPrintSetup.getFitWidth());
			setup.setPaperSize(inputExcelPrintSetup.getPaperSize());
			sheet.setFitToPage(inputExcelPrintSetup.isFitToPage());
			sheet.setPrintRowAndColumnHeadings(inputExcelPrintSetup.isPrintRowAndColumnHeadings());
			sheet.setHorizontallyCenter(inputExcelPrintSetup.isHorizontallyCenter());
			sheet.setVerticallyCenter(inputExcelPrintSetup.isVerticallyCenter());
		}
		sheet.setMargin(Sheet.TopMargin, 0.75);
		sheet.setMargin(Sheet.RightMargin, 0.70);
		sheet.setMargin(Sheet.BottomMargin, 0.75);
		sheet.setMargin(Sheet.LeftMargin, 0.70);
		sheet.setMargin(Sheet.HeaderMargin, 0.30);
		sheet.setMargin(Sheet.FooterMargin, 0.30);
	}

}
