package org.jeecg.modules.tools.word;

import cn.hutool.core.io.IoUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.util.Units;
import org.apache.poi.xwpf.usermodel.*;
import org.jeecg.common.util.SpringContextHolder;
import org.jeecg.modules.tools.CommonUtil;
import org.jeecg.modules.tools.FileUtils;
import org.jeecg.modules.tools.Global;
import org.jeecg.modules.tools.IdGen;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 对word模板的操作，包含替换文本和列表向下扩展两种<br>
 * 本工具类的使用需要配合ExportWord.class，具体Demo看ExportWord类即可
 * @author byx
 */
public class WordOperator {

	private static Logger logger = LoggerFactory.getLogger(WordOperator.class);
	
	/**
	 * 模板替换标签开头
	 */
	private final static String LABEL_STR = "{";
	/**
	 * 模板替换标签结尾
	 */
	private final static String LABEL_END = "}";
	/**
	 * 模板替换图片标签开头(List)
	 */
	private final static String LABEL_IMAGE_STR = "<";
	/**
	 * 模板替换图片标签结尾(List)
	 */
	private final static String LABEL_IMAGE_END = ">";
	
	/**
	 * 模板替换图片标签开头(List)
	 */
	private final static String LABEL_RICH_TEXT_STR = "<richText[";
	/**
	 * 模板替换图片标签结尾(List)
	 */
	private final static String LABEL_RICH_TEXT_END = "]richText>";

	/**
	 * 加载word对象
	 */
	private XWPFDocument doc;
	
	public XWPFDocument getDoc() {
        return doc;
    }

    public void setDoc(XWPFDocument doc) {
        this.doc = doc;
    }



    private int picturesSize = 0;
	
	public int getPicturesSize() {
		if(picturesSize == 0){
			picturesSize = doc.getAllPictures().size();
		}
		return picturesSize;
	}

	public void setPicturesSize(int picturesSize) {
		this.picturesSize = picturesSize;
	}

	/**
	 * 构造方法，把模板文档加载进来<br>
	 * 		因为微软只提供XWPFRun对象替换文本，而它在切割段落文本的时候却是非常没有规律的，故而本方法要求模板必须遵循一定的规范来编写：<br>
	 * 		１．模板里除了标签外，其余地方禁止使用"{"和"}"字符串；<br>
	 * 		２．不允许出现"{}"这种写法，即标签必须包含key；
	 */
	public WordOperator(InputStream is) throws Exception {
		doc = new XWPFDocument(is);
	}
	
	

	/**
	 * 要替换标签的KV对
	 */
	private Map<String, Object> replaceMap;

	public Map<String, Object> getReplaceMap() {
		return replaceMap;
	}

	public void setReplaceMap(Map<String, Object> replaceMap) {
		this.replaceMap = replaceMap;
	}

	/**
	 * 
	 * 功能说明	: 创建图片，根据段落
	 * 创建者		: byx
	 * 修改日期	: 2018年9月15日
	 * @param pic	图片位置
	 * @param picPath	图片路径
	 * @param width	图片宽度
	 * @param height	图片高度
	 * @param picAttch	图片后缀文字
	 */
	public void createPicture(XWPFParagraph pic, String picPath, int width, int height, String picAttch){
		pic.setAlignment(ParagraphAlignment.CENTER);

		XWPFRun run = pic.createRun();

		try {
			if(StringUtils.isBlank(picPath) || picPath.lastIndexOf(".") == -1){
				logger.error("文件路径错误:");
				logger.error(picPath);
				return;
			}
			//根据文件后缀名设置图片格式
			String picType = picPath.substring(picPath.lastIndexOf("\\.")+1);
			int res = XWPFDocument.PICTURE_TYPE_PICT;
			if(picType != null){
				if(picType.equalsIgnoreCase("png")){
					res = XWPFDocument.PICTURE_TYPE_PNG;
				}else if(picType.equalsIgnoreCase("gif")) {
					res = XWPFDocument.PICTURE_TYPE_GIF;
				}else if(picType.equalsIgnoreCase("emf")){
					res = XWPFDocument.PICTURE_TYPE_EMF;
				}else if(picType.equalsIgnoreCase("jpg") || picType.equalsIgnoreCase("jpeg")){
					res = XWPFDocument.PICTURE_TYPE_JPEG;
				}else if(picType.equalsIgnoreCase("wmf")){
					res = XWPFDocument.PICTURE_TYPE_WMF;
				}else if(picType.equalsIgnoreCase("pict")){
					res = XWPFDocument.PICTURE_TYPE_PICT;
				}else if(picType.equalsIgnoreCase("dib")){
					res = XWPFDocument.PICTURE_TYPE_DIB;
				}else if(picType.equalsIgnoreCase("tiff")){
					res = XWPFDocument.PICTURE_TYPE_TIFF;
				}else if(picType.equalsIgnoreCase("eps")){
					res = XWPFDocument.PICTURE_TYPE_EPS;
				}else if(picType.equalsIgnoreCase("bmp")){
					res = XWPFDocument.PICTURE_TYPE_BMP;
				}else if(picType.equalsIgnoreCase("wpg")){
					res = XWPFDocument.PICTURE_TYPE_WPG;
				}
			}
			//塞进文档中
//			doc.addPictureData(new FileInputStream(picPath),res);
			if(StringUtils.isNotBlank(picAttch)){
				run.setText(picAttch);
			}
			//补充完整路径
			Global global = SpringContextHolder.getBean(Global.class);
			InputStream is = new FileInputStream(global.uploadPath + "/" + picPath);
//			run.addPicture(is,res,picPath,Units.pixelToEMU(width), Units.pixelToEMU(height));
			run.addPicture(is,res,picPath,Units.toEMU(width), Units.toEMU(height));
//			run.addPicture(is,res,picPath,width, height);
			IoUtil.close(is);
		} catch (InvalidFormatException e) {
			logger.error(e.toString());
			e.printStackTrace();
		} catch (IOException e) {
			logger.error("没有找到相应的文件！");
			logger.error(e.toString());
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * 功能说明	: 多个图片支持（|分隔）
	 * 创建者		: byx
	 * 修改日期	: 2018年9月26日
	 * @param pic	图片位置
	 * @param picPath	图片路径
	 * @param width	图片宽度
	 * @param height	图片高度
	 * @param picAttch	图片后缀文字
	 */
	public void createPicturePlus(XWPFParagraph pic, String picPath, int width, int height, String picAttch){
		if(StringUtils.isBlank(picPath)){
			logger.warn("文件路径错误:");
			logger.warn(picPath);
			return;
		}
		if(StringUtils.indexOf(picPath, ",") == -1){
			//为单个时直接插入图片
			createPicture(pic,picPath,width,height,picAttch);
		}else{
			//多个时，
			String[] paths = StringUtils.split(picPath,",");
			for (String s : paths) {
				createPicture(pic,s,width,height,picAttch);
			}
		}
//		createPicture
	}

	/**
	 * 在word模板文档里，用{key}标签来表示要替换的内容，把key对应的value存储于replaceMap里，即可完成替换
	 */
	public WordOperator replaceText(Map<String, Object> replaceMap) {
		this.replaceMap = replaceMap;
		// 替换文本内容的标签
		List<XWPFParagraph> paragraphList = doc.getParagraphs();
		replaceText(paragraphList);
		// 替换表格内部标签
		List<XWPFTable> tableList = doc.getTables();
		for (XWPFTable table : tableList) {
			for (int i = 0; i < table.getNumberOfRows(); i++) {
				XWPFTableRow row = table.getRow(i);
				List<XWPFTableCell> tableCellList = row.getTableCells();
				for (XWPFTableCell cell : tableCellList) {
					replaceText(cell.getParagraphs());
				}
			}
		}
		return this;
	}

	/**
	 * 在word模板文档里，用{key}标签来表示要替换的内容，把key对应的value存储于replaceMap里，即可完成替换（带有替换图片的功能<image>）
	 */
	public WordOperator replaceTextPlus(Map<String, Object> replaceMap) {
		this.replaceMap = replaceMap;
		// 替换文本内容的标签
		List<XWPFParagraph> paragraphList = doc.getParagraphs();
		replaceTextPlus(paragraphList);
		// 替换表格内部标签
		List<XWPFTable> tableList = doc.getTables();
		for (XWPFTable table : tableList) {
			for (int i = 0; i < table.getNumberOfRows(); i++) {
				XWPFTableRow row = table.getRow(i);
				List<XWPFTableCell> tableCellList = row.getTableCells();
				for (XWPFTableCell cell : tableCellList) {
					replaceTextPlus(cell.getParagraphs());
				}
			}
		}
		return this;
	}


	public WordOperator replaceTextPlusRemove(Map<String, Object> replaceMap,String removeList) {
		this.replaceMap = replaceMap;
		// 替换文本内容的标签
		List<XWPFParagraph> paragraphList = doc.getParagraphs();
		replaceTextPlus(paragraphList);
		// 替换表格内部标签
		List<XWPFTable> tableList = doc.getTables();
		for (XWPFTable table : tableList) {
			for (int i = 0; i < table.getNumberOfRows(); i++) {
				XWPFTableRow row = table.getRow(i);
				List<XWPFTableCell> tableCellList = row.getTableCells();
				for (XWPFTableCell cell : tableCellList) {
					replaceTextPlus(cell.getParagraphs());
				}
			}
		}
		return this;
	}

	/**
	 * 在word模板文档里，用{key}标签来表示要替换的内容，把key对应的value存储于replaceMap里，即可完成替换（带图片，仅支持一个图片）
	 */
	public WordOperator replaceTextAndCreatePicture(Map<String, Object> replaceMap) {
		this.replaceMap = replaceMap;
		// 替换文本内容的标签
		List<XWPFParagraph> paragraphList = doc.getParagraphs();
		replaceText(paragraphList);
		// 替换表格内部标签
		List<XWPFTable> tableList = doc.getTables();
		for (XWPFTable table : tableList) {
			for (int i = 0; i < table.getNumberOfRows(); i++) {
				XWPFTableRow row = table.getRow(i);
				List<XWPFTableCell> tableCellList = row.getTableCells();
				for (XWPFTableCell cell : tableCellList) {
					replaceTextAndCreatePicture(cell.getParagraphs());
				}
			}
		}
		return this;
	}
	
	/**
	 * 把替换段落抽取出一个方法，在替换文本和替换表格里都可以调用
	 */
	private void replaceText(List<XWPFParagraph> paragraphList) {
		// 取出word模板里的全部段落，遍历
		for (int i = 0; i < paragraphList.size(); i++) {
			XWPFParagraph paragraph = paragraphList.get(i);
			// 拿出每一个段落，判断内容里是否包含字符串"{"和"}"，只有两者同时存在了才执行替换标签的逻辑
			String paragraphText = paragraph.getText();
			if (!StringUtils.isBlank(paragraphText) && paragraphText.indexOf(LABEL_STR) != -1 && paragraphText.indexOf(LABEL_END) != -1) {
				// 每一个段落分很多小段文本，官方API只能否通过XWPFRun对象执行替换文本功能
				List<XWPFRun> runList = paragraph.getRuns();
				// 组装replaceMap的key
				String key = "";
				// 是否检测到有"{"字符串，有为true，没有为false
				boolean include = false;
				for (int j = 0; j < runList.size(); j++) {
					XWPFRun run = runList.get(j);
					String text = run.getText(0);
					// 取出每一个小段里标签头和尾的角标
					int labelStrIndex = StringUtils.indexOf(text, LABEL_STR);
					int labelEndIndex = StringUtils.indexOf(text, LABEL_END);
					// 只有{
					if (labelStrIndex != -1 && labelEndIndex == -1) {
						// 例如有"aaa{bbb"，则aaa不动，{bbb去掉，同时把bbb累加到key里
						String textBefore = text.substring(0, labelStrIndex);
						String textAfter = text.substring(labelStrIndex + 1);
						run.setText(textBefore, 0);
						key += textAfter;
						include = true;
						// 只有}
					} else if (labelStrIndex == -1 && labelEndIndex != -1) {
						// 例如有"aaa}bbb"，则aaa}去掉，bbb不动，同时把aaa累加到key里
						String textBefore = text.substring(0, labelEndIndex);
						String textAfter = text.substring(labelEndIndex + 1);
						key += textBefore;
						Object value = replaceMap.get(key);
						if (value == null || StringUtils.isBlank(value.toString())) {
							value = "";
						}
						key = "";
						run.setText(value + textAfter, 0);
						include = false;
						// 两个都没有
					} else if (labelStrIndex == -1 && labelEndIndex == -1) {
						// 例如有"aaa"，如果之前有{了，则把内容累加到key里，同时去掉内容；如果没有{，则说明是普通文本，跳过不管
						if (include) {
							key += text;
							run.setText("", 0);
						}
						// 两个都存在，这种情况比较复杂。经过多次试验，XWPFRun切割内容，只会同时各出现1次而已
					} else {
						// {在前，}在后
						if (labelStrIndex < labelEndIndex) {
							// 例如有"aaa{bbb}ccc"，则aaa不动，{bbb}去掉同时进行替换，ccc保留（因为有规范，所以这是一个独立完整的标签，且bbb绝对有值）
							String textBefore = text.substring(0, labelStrIndex);
							String textMiddle = text.substring(labelStrIndex + 1, labelEndIndex);
							String textAfter = text.substring(labelEndIndex + 1);
							Object value = replaceMap.get(textMiddle);
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText(textBefore + value + textAfter, 0);
							key = "";
							include = false;
							// }在前，{在后
						} else {
							// 例如有"aaa}bbb{ccc"，则aaa}去掉，且进行替换；bbb不动，{ccc去掉，累加到key里去
							String textBefore = text.substring(0, labelEndIndex);
							String textMiddle = text.substring(labelEndIndex + 1, labelStrIndex);
							String textAfter = text.substring(labelStrIndex + 1);
							key += textBefore;
							Object value = replaceMap.get(key);
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText(value + textMiddle, 0);
							key = textAfter;
							include = true;
						}
					}
				}
			}
		}
	}
	/**
	 * 把替换段落抽取出一个方法，在替换文本和替换表格里都可以调用（可以替换为图片）
	 */
	private void replaceTextPlus(List<XWPFParagraph> paragraphList) {
		//InputStream imageFileIs = null;
		// 取出word模板里的全部段落，遍历
		for (int i = 0; i < paragraphList.size(); i++) {
			XWPFParagraph paragraph = paragraphList.get(i);
			// 拿出每一个段落，判断内容里是否包含字符串"{"和"}"，只有两者同时存在了才执行替换标签的逻辑
			String paragraphText = paragraph.getText();
			if (!StringUtils.isBlank(paragraphText) && StringUtils.indexOf(paragraphText,LABEL_STR) != -1 && StringUtils.indexOf(paragraphText,LABEL_END) != -1) {
				// 每一个段落分很多小段文本，官方API只能通过XWPFRun对象执行替换文本功能
				List<XWPFRun> runList = paragraph.getRuns();
				// 组装replaceMap的key
				String key = "";
				// 是否检测到有"{"字符串，有为true，没有为false
				boolean include = false;
				for (int j = 0; j < runList.size(); j++) {
					XWPFRun run = runList.get(j);
					//颜色
//					run.setColor("F3C917");
					//加粗
//					run.setBold(true);
					//背景色
//					run.getCTR().addNewRPr().addNewHighlight().setVal(STHighlightColor.YELLOW);
					//底纹
//					CTShd shd = run.getCTR().addNewRPr().addNewShd();
//					shd.setFill("FFFF00");

					String text = run.getText(0);
					// 取出每一个小段里标签头和尾的角标
					//是否存在两个{
					boolean isParagraphReplaceAllText = false;
					if(StringUtils.ordinalIndexOf(text,LABEL_STR,2) != -1){
						isParagraphReplaceAllText = true;
						text = paragraphReplaceAllText(text,replaceMap);
					}
					int labelStrIndex = StringUtils.indexOf(text, LABEL_STR);
					int labelEndIndex = StringUtils.indexOf(text, LABEL_END);
					// 只有{
					if (labelStrIndex != -1 && labelEndIndex == -1) {
						// 例如有"aaa{bbb"，则aaa不动，{bbb去掉，同时把bbb累加到key里
						String textBefore = text.substring(0, labelStrIndex);
						String textAfter = text.substring(labelStrIndex + 1);
						run.setText(textBefore, 0);
						key += textAfter;
						include = true;
						// 只有}
					} else if (labelStrIndex == -1 && labelEndIndex != -1) {
						// 例如有"aaa}bbb"，则aaa}去掉，bbb不动，同时把aaa累加到key里
						String textBefore = text.substring(0, labelEndIndex);
						String textAfter = text.substring(labelEndIndex + 1);
						key += textBefore;
						//Object value = replaceMap.get(key);
						String value = (String)replaceMap.get(key);
						if (StringUtils.isBlank(value)) value = "";
						WordTextStyle style = (WordTextStyle)replaceMap.get(key + "_style");
						//添加样式
						if(style != null) style.run(run);
						if (StringUtils.isBlank(value)) value = "";
						value = StringEscapeUtils.unescapeHtml4(value);
						key = "";
						//--2020-04修改
						String[] values = value.split("\n");
						if(values.length > 1) {
							run.setText(values[0],0);
							for (int x = 1; x < values.length; x++) {
								//存在分段则新建一个run
								XWPFRun newrun = paragraph.createRun();
								//copy样式
								newrun.getCTR().setRPr(run.getCTR().getRPr());
								//换行
								newrun.addBreak();
								//缩进
								//newrun.addTab();
								if(x == values.length - 1) {
									newrun.setText(values[x] + textAfter);
								}else {
									newrun.setText(values[x]);
								}
							}
						}else {
							run.setText(value + textAfter, 0);
						}
						include = false;
						// 两个都没有
					} else if (labelStrIndex == -1 && labelEndIndex == -1) {
						// 例如有"aaa"，如果之前有{了，则把内容累加到key里，同时去掉内容；如果没有{，则说明是普通文本，跳过不管
						if (include) {
							key += text;
							run.setText("", 0);
						}
						if(isParagraphReplaceAllText){
							//替换文本
							run.setText(text,0);
						}
						// 两个都存在，这种情况比较复杂。经过多次试验，XWPFRun切割内容，只会同时各出现1次而已
					} else {
						// {在前，}在后
						if (labelStrIndex < labelEndIndex) {
							// 例如有"aaa{bbb}ccc"，则aaa不动，{bbb}去掉同时进行替换，ccc保留（因为有规范，所以这是一个独立完整的标签，且bbb绝对有值）
							String textBefore = text.substring(0, labelStrIndex);
							String textMiddle = text.substring(labelStrIndex + 1, labelEndIndex);
							String textAfter = text.substring(labelEndIndex + 1);
							String value = (String)replaceMap.get(textMiddle);
							if (StringUtils.isBlank(value)) value = "";
							WordTextStyle style = (WordTextStyle)replaceMap.get(textMiddle + "_style");
							//添加样式
							if(style != null) style.run(run);
							value = StringEscapeUtils.unescapeHtml4(value);
							//--2020-04修改
							String[] values = value.split("\n");
							if(values.length > 1) {
								run.setText(textBefore + values[0],0);
								for (int x = 1; x < values.length; x++) {
								   //存在分段则新建一个run
									XWPFRun newrun = paragraph.createRun();
									//copy样式
									newrun.getCTR().setRPr(run.getCTR().getRPr());
									//换行
									newrun.addBreak();
									//缩进
									//newrun.addTab();
									if(x == values.length - 1) {
										newrun.setText(values[x] + textAfter,0);
									}else {
										newrun.setText(values[x]);
									}
								}
							}else {
								run.setText(textBefore + value + textAfter, 0);
							}

							key = "";
							include = false;
							// }在前，{在后
						} else {
							// 例如有"aaa}bbb{ccc"，则aaa}去掉，且进行替换；bbb不动，{ccc去掉，累加到key里去
							String textBefore = text.substring(0, labelEndIndex);
							String textMiddle = text.substring(labelEndIndex + 1, labelStrIndex);
							String textAfter = text.substring(labelStrIndex + 1);
							key += textBefore;
							String value = (String)replaceMap.get(key);
							if (value == null || StringUtils.isBlank(value.toString())) value = "";
							WordTextStyle style = (WordTextStyle)replaceMap.get(textMiddle + "_style");
							//添加样式
							if(style != null) style.run(run);
							value = StringEscapeUtils.unescapeHtml4(value);
							//--2020-04修改
							String[] values = value.toString().split("\n");
							if(values.length > 1) {
								run.setText(values[0],0);
								for (int x = 1; x < values.length; x++) {
								   //存在分段则新建一个run
									XWPFRun newrun = paragraph.createRun();
									//copy样式
									newrun.getCTR().setRPr(run.getCTR().getRPr());
									//换行
									newrun.addBreak();
									//缩进
									//newrun.addTab();
									if(x == values.length - 1) {
										newrun.setText(values[x] + textMiddle,0);
									}else {
										newrun.setText(values[x]);
									}
								}
							}else {
								run.setText(value + textMiddle, 0);
							}
							key = textAfter;
							include = true;
						}
					}
				}
			//富文本标签（转成图片的富文本）
			} else if(StringUtils.isNotBlank(paragraphText) && paragraphText.indexOf(LABEL_RICH_TEXT_STR) != -1 && paragraphText.indexOf(LABEL_RICH_TEXT_END) != -1){
				System.out.println("paragraphText="+paragraphText);
				//如果匹配到‘<’和‘>’则开始替换为图片
				// 每一个段落分很多小段文本，官方API只能否通过XWPFRun对象执行替换文本功能
				List<XWPFRun> runList = paragraph.getRuns();
				//组装replaceMap的key
				String key = "";
				
				//<richText[课题设计论证]richText>
				//摘出key
				int $labelStrIndex = StringUtils.indexOf(paragraphText, LABEL_RICH_TEXT_STR);
				int $labelEndIndex = StringUtils.indexOf(paragraphText, LABEL_RICH_TEXT_END);
				
				//必须存在开始和结束标签
				if($labelStrIndex != -1 && $labelEndIndex != -1){
					key = paragraphText;
					key = key.substring($labelStrIndex + LABEL_RICH_TEXT_STR.length());
					$labelEndIndex = StringUtils.indexOf(key, LABEL_RICH_TEXT_END);
					key = key.substring(0,$labelEndIndex);
				}
						
				//去除整行（整个paragraph）
				for (int j = 0; j < runList.size(); j++) {
					XWPFRun run = runList.get(j);
					run.setText("", 0);
				}
				//添加图片
				//创建一个run
				//XWPFRun $run = paragraph.createRun();
				Object value = replaceMap.get(key);
				if (value == null || StringUtils.isBlank(value.toString())) {
					value = "";
				}
				String imagePath = getHtmlImage(value.toString());
				//try {
					//imageFileIs = new FileInputStream(new File(imagePath));
					//$run.addPicture(imageFileIs, 6, "image", 595,842);
				//} catch (InvalidFormatException | IOException e) {
					//logger.error("图片读取或添加错误！");
					//logger.error(e.getMessage(),e);
					//e.printStackTrace();
				//}
				createPicturePlus(paragraph,imagePath,595,842,"");
				//FileUtils.deleteFile(imagePath);
				//$run.setText();
			//图片标签
		 	}else if(StringUtils.isNotBlank(paragraphText) && paragraphText.indexOf(LABEL_IMAGE_STR) != -1 && paragraphText.indexOf(LABEL_IMAGE_END) != -1){
				//如果匹配到‘<’和‘>’则开始替换为图片
				// 每一个段落分很多小段文本，官方API只能否通过XWPFRun对象执行替换文本功能
				List<XWPFRun> runList = paragraph.getRuns();
				// 组装replaceMap的key
				String key = "";
				//读取简易标签对象如：‘image,10,10,后缀’
				WordImageLabel wordImageLabel = null;
				// 是否检测到有"<"字符串，有为true，没有为false
				boolean include = false;
				for (int j = 0; j < runList.size(); j++) {
					XWPFRun run = runList.get(j);
					String text = run.getText(0);
					// 取出每一个小段里标签头和尾的角标
					int labelStrIndex = StringUtils.indexOf(text, LABEL_IMAGE_STR);
					int labelEndIndex = StringUtils.indexOf(text, LABEL_IMAGE_END);
					// 只有<
					if (labelStrIndex != -1 && labelEndIndex == -1) {
						// 例如有"aaa<bbb"，则aaa不动，<bbb去掉，同时把bbb累加到key里
//						String textBefore = text.substring(0, labelStrIndex);
						String textAfter = text.substring(labelStrIndex + 1);
						run.setText("", 0);
//						run.setText(textBefore, 0);
						key += textAfter;
						include = true;
						// 只有}
					} else if (labelStrIndex == -1 && labelEndIndex != -1) {
						// 例如有"aaa>bbb"，则aaa>去掉，bbb不动，同时把aaa累加到key里
						String textBefore = text.substring(0, labelEndIndex);
//						String textAfter = text.substring(labelEndIndex + 1);
						key += textBefore;
						//读取简易标签对象
						wordImageLabel = new WordImageLabel(key);
						Object value = replaceMap.get(wordImageLabel.getKey());
						if (value == null || StringUtils.isBlank(value.toString())) {
							value = "";
						}
						key = "";
						//创建图片
						run.setText("", 0);
//						run.setText(value + textAfter, 0);
						createPicturePlus(paragraph,(String)value,wordImageLabel.getWidth(),wordImageLabel.getHeight(),wordImageLabel.getPicAttch());
						include = false;
						// 两个都没有
					} else if (labelStrIndex == -1 && labelEndIndex == -1) {
						// 例如有"aaa"，如果之前有<了，则把内容累加到key里，同时去掉内容；如果没有<，则说明是普通文本，跳过不管
						if (include) {
							key += text;
							run.setText("", 0);
						}
						// 两个都存在，这种情况比较复杂。经过多次试验，XWPFRun切割内容，只会同时各出现1次而已
					} else {
						// <在前，>在后
						if (labelStrIndex < labelEndIndex) {
							// 例如有"aaa<bbb>ccc"，则aaa不动，<bbb>去掉同时进行替换，ccc保留（因为有规范，所以这是一个独立完整的标签，且bbb绝对有值）
//							String textBefore = text.substring(0, labelStrIndex);
							String textMiddle = text.substring(labelStrIndex + 1, labelEndIndex);
//							String textAfter = text.substring(labelEndIndex + 1);
							wordImageLabel = new WordImageLabel(textMiddle);
							Object value = replaceMap.get(wordImageLabel.getKey());
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText("", 0);
//							run.setText(textBefore + value + textAfter, 0);
							createPicturePlus(paragraph,(String)value,wordImageLabel.getWidth(),wordImageLabel.getHeight(),wordImageLabel.getPicAttch());
							key = "";
							include = false;
							// >在前，<在后
						} else {
							// 例如有"aaa>bbb<ccc"，则aaa>去掉，且进行替换；bbb不动，<ccc去掉，累加到key里去
							String textBefore = text.substring(0, labelEndIndex);
//							String textMiddle = text.substring(labelEndIndex + 1, labelStrIndex);
							String textAfter = text.substring(labelStrIndex + 1);
							key += textBefore;
							wordImageLabel = new WordImageLabel(key);
							Object value = replaceMap.get(wordImageLabel.getKey());
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText("", 0);
							createPicturePlus(paragraph,(String)value,wordImageLabel.getWidth(),wordImageLabel.getHeight(),wordImageLabel.getPicAttch());
							key = textAfter;
							include = true;
						}
					}
				}				
			}
			
			
		}
	}

	/**
	 * 替换一段文字里有多个{key1},{key2}
	 * @param str
	 * @param replaceMap
	 * @return
	 */
	private String paragraphReplaceAllText(String str, Map<String, Object> replaceMap){
		return CommonUtil.getReplaceMapValue(str,replaceMap,LABEL_STR,LABEL_END);
	}

	/**
	 * 把替换段落抽取出一个方法，在替换文本和替换表格里都可以调用(带图片)，只替换{IMAGE}
	 */
	private void replaceTextAndCreatePicture(List<XWPFParagraph> paragraphList) {
		// 取出word模板里的全部段落，遍历
		for (int i = 0; i < paragraphList.size(); i++) {
			XWPFParagraph paragraph = paragraphList.get(i);
			// 拿出每一个段落，判断内容里是否包含字符串"{"和"}"，只有两者同时存在了才执行替换标签的逻辑
			String paragraphText = paragraph.getText();
			if (!StringUtils.isBlank(paragraphText)) {
				//判断是否为唯一的image关键字
				if(StringUtils.equals(paragraphText, "{IMAGE}") && paragraphText.indexOf(LABEL_STR) != -1 && paragraphText.indexOf(LABEL_END) != -1){
					//去掉关键字标记
					List<XWPFRun> runList = paragraph.getRuns();
					for (int j = 0; j < runList.size(); j++) {
						XWPFRun run = runList.get(j);
						run.setText("", 0);
					}
					//获取图片地址
					String picPath = (String) replaceMap.get("IMAGE");
					//创建图片
					createPicture(paragraph,picPath,83,83," ");
					//创建后不需要普通的文字替换了，跳过本段落
					continue;
				}
				// 每一个段落分很多小段文本，官方API只能否通过XWPFRun对象执行替换文本功能
				List<XWPFRun> runList = paragraph.getRuns();
				// 组装replaceMap的key
				String key = "";
				// 是否检测到有"{"字符串，有为true，没有为false
				boolean include = false;
				for (int j = 0; j < runList.size(); j++) {
					XWPFRun run = runList.get(j);
					String text = run.getText(0);
					// 取出每一个小段里标签头和尾的角标
					int labelStrIndex = text.indexOf(LABEL_STR);
					int labelEndIndex = text.indexOf(LABEL_END);
					// 只有{
					if (labelStrIndex != -1 && labelEndIndex == -1) {
						// 例如有"aaa{bbb"，则aaa不动，{bbb去掉，同时把bbb累加到key里
						String textBefore = text.substring(0, labelStrIndex);
						String textAfter = text.substring(labelStrIndex + 1);
						run.setText(textBefore, 0);
						//createPicture(paragraph,"");
						key += textAfter;
						include = true;
						// 只有}
					} else if (labelStrIndex == -1 && labelEndIndex != -1) {
						// 例如有"aaa}bbb"，则aaa}去掉，bbb不动，同时把aaa累加到key里
						String textBefore = text.substring(0, labelEndIndex);
						String textAfter = text.substring(labelEndIndex + 1);
						key += textBefore;
						Object value = replaceMap.get(key);
						if (value == null || StringUtils.isBlank(value.toString())) {
							value = "";
						}
						key = "";
						run.setText(value + textAfter, 0);
						include = false;
						// 两个都没有
					} else if (labelStrIndex == -1 && labelEndIndex == -1) {
						// 例如有"aaa"，如果之前有{了，则把内容累加到key里，同时去掉内容；如果没有{，则说明是普通文本，跳过不管
						if (include) {
							key += text;
							run.setText("", 0);
						}
						// 两个都存在，这种情况比较复杂。经过多次试验，XWPFRun切割内容，只会同时各出现1次而已
					} else {
						// {在前，}在后
						if (labelStrIndex < labelEndIndex) {
							// 例如有"aaa{bbb}ccc"，则aaa不动，{bbb}去掉同时进行替换，ccc保留（因为有规范，所以这是一个独立完整的标签，且bbb绝对有值）
							String textBefore = text.substring(0, labelStrIndex);
							String textMiddle = text.substring(labelStrIndex + 1, labelEndIndex);
							String textAfter = text.substring(labelEndIndex + 1);
							Object value = replaceMap.get(textMiddle);
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText(textBefore + value + textAfter, 0);
							key = "";
							include = false;
							// }在前，{在后
						} else {
							// 例如有"aaa}bbb{ccc"，则aaa}去掉，且进行替换；bbb不动，{ccc去掉，累加到key里去
							String textBefore = text.substring(0, labelEndIndex);
							String textMiddle = text.substring(labelEndIndex + 1, labelStrIndex);
							String textAfter = text.substring(labelStrIndex + 1);
							key += textBefore;
							Object value = replaceMap.get(key);
							if (value == null || StringUtils.isBlank(value.toString())) {
								value = "";
							}
							run.setText(value + textMiddle, 0);
							key = textAfter;
							include = true;
						}
					}
				}
			}
		}
	}
	
//	public static void main(String[] args) {
//		String text = "<richText[课题设计论证]richText>";
//		//摘出key
//		int $labelStrIndex = StringUtils.indexOf(text, LABEL_RICH_TEXT_STR);
//		int $labelEndIndex = StringUtils.indexOf(text, LABEL_RICH_TEXT_END);
//
//
//		System.out.println("$labelStrIndex=" + $labelStrIndex);
//		System.out.println("$labelEndIndex=" + $labelEndIndex);
//		//必须存在开始和结束标签
//		if($labelStrIndex != -1 && $labelEndIndex != -1){
//			String key = text;
//			key = key.substring($labelStrIndex + LABEL_RICH_TEXT_STR.length());
//			$labelEndIndex = StringUtils.indexOf(key, LABEL_RICH_TEXT_END);
//			System.out.println("key="+key);
//			key = key.substring(0,$labelEndIndex);
//			System.out.println("key="+key);
//
//		}
//
//		HtmlImageGenerator imageGenerator = new HtmlImageGenerator();
////		String htmlstr = "<div><img src='http://avatar.csdn.net/2/3/A/3_u010304626.jpg'><p>编号:10001</p></div>";
//		String htmlstr = "<div><img src='http://car0.autoimg.cn/upload/spec/8090/u_20101129100303187264.jpg'><p>编号:10001</p></div>";
////		String htmlstr = "<img src=\"http://127.0.0.1:9000/education/static/uploadify/cancel.png\" />";
////		htmlstr = htmlstr.replaceAll("/education/", "http://127.0.0.1:9000/education/");
//		imageGenerator.loadHtml(htmlstr);
////		imageGenerator.loadUrl("https://www.baidu.com");
////		imageGenerator.setSize(new Dimension(595,842));
//		try {
//			imageGenerator.getBufferedImage();
//			Thread.sleep(4000);
//		} catch (InterruptedException e) {
//			e.printStackTrace();
//		}
//		imageGenerator.saveAsImage("z:/baidu.png");
//	}
	
	public static void main0(String[] args) {
		getHtmlImage("aa");
	}
	public static String getHtmlImage(String html){
//		String path = Global.getConfig("userfiles.basedir") + "/htmlImageTemp/";
		String path = "";
		FileUtils.createDirectory(path);
		path += IdGen.uuid() + ".png";
		getHtmlImage(html,path);
		return path;
	}
	public static void getHtmlImage(String html, String path){
		getHtmlImage(html,8000,path);
	}
	public static void getHtmlImage(String html, int sleep, String path){
//		HtmlImageGenerator imageGenerator = new HtmlImageGenerator();
//		imageGenerator.loadHtml(html);
//		imageGenerator.setSize(new Dimension(595,842));
//		try {
//			imageGenerator.getBufferedImage();
//			Thread.sleep(sleep);
//		} catch (InterruptedException e) {
//			e.printStackTrace();
//		}
//		imageGenerator.saveAsImage(path);
	}
	/**
	 * 表格向下扩展数据行，因为POI对word格式支持较差，故我们做一些约定<br>
	 * 	１．约定表格要有一行模板行，用于扩展数据行的样式效仿；<br>
	 * 	２．模板表格里最好不要有合并行，如非常需要，则可以用隐藏单元格或设置白色边框的方式巧妙替代合并行；<br>
	 * 	３．模板行下方不允许出现其他行<br>
	 * @param tableIndex	表格角标，第1个表为0
	 * @param tmpRowIndex		模板行角标，第1行为0（模板行不是标题行，而是扩展数据行样式的效仿对象）
	 * @param isDelTmpRow	true-删除模板行；false-不删除模板行
	 * @param rowlist		要插入的数据集合，格式为［行集合<列集合>，列集合的长度最好与模板行长度一致］
	 */
	public WordOperator insert2Table(int tableIndex, int tmpRowIndex, boolean isDelTmpRow, List<List<String>> rowlist) {
		// 因为需要对rowlist进行添加操作，故而我们重新做一个变量，防止因为改变而产生问题
		List<List<String>> dataList = new ArrayList<>(rowlist);
		// 根据角标取出表格对像
		XWPFTable table = doc.getTables().get(tableIndex);
		// 再根据行号获取模板行
		XWPFTableRow tmpRow = table.getRow(tmpRowIndex);
		// 不知道是什么原因，最后一列数据会跑到模板行上一行，故而这里在最末添加一组数据，解决这个bug
		dataList.add(new ArrayList<>());
		// 遍历数据集合
		for (int i = 0, len = dataList.size(); i < dataList.size(); i++) {
			// 计算数据行编号
			int dataIndex = tmpRowIndex + 1 + i;
			// 按照模板行样式添加一行到数据行
			table.addRow(tmpRow, dataIndex);
			if (i < len - 1) {
				// 取出这一行
				XWPFTableRow row = table.getRow(dataIndex);
				// 取出这一行全部单元格
				List<XWPFTableCell> cellList = row.getTableCells();
				// 取出这一行对应的数据，数据与单元格角标位置是一致的，一一替换文本
				List<String> colList = dataList.get(i);
				for (int j = 0; j < cellList.size(); j++) {
//					if(StringUtils.contains(colList.get(j),"\n")){
//						setCellMultiLineText(cellList.get(j), colList, j);
//					}else{
						setCellText(cellList.get(j), colList, j);
//					}
				}
			}
		}
		// 删掉那一行为了bug而添加的空白行（因为空白行跑到模板行上一行，所以tmpRowIndex为该行角标）
		table.removeRow(tmpRowIndex);
		// 删除模版行（因为上面已经删除了空白行了，所以tmpRowIndex就变成了下一行模板行了）
		if (isDelTmpRow) {
			table.removeRow(tmpRowIndex);
		}
		return this;
	}

	/**
	 * 替换单元格文本
	 * @param cell        单元格
	 * @param colList    替换文本的集合（角标越位了，就置空）
	 * @param j            单元格角标，也就是文本集合的角标
	 */
	private void setCellText(XWPFTableCell cell, List<String> colList, int j) {
//		cell.setColor("FF0000");
		//　无论一个单元格被切割成多少个run，只需要修改第一个即可，改完就return掉了
		List<XWPFParagraph> paragraphs = cell.getParagraphs();
		for (XWPFParagraph p : paragraphs) {
			List<XWPFRun> runs = p.getRuns();
			for (XWPFRun r : runs) {
				try {
					r.setText(colList.get(j), 0);
					//染上背景色
//					r.getCTR().addNewRPr().addNewHighlight().setVal(STHighlightColor.YELLOW);
				} catch (Exception e) {
					r.setText("", 0);
				}
				return;
			}
		}
	}


	/**
	 * 替换单元格文本
	 * @param cell        单元格
	 * @param colList    替换文本的集合（角标越位了，就置空）
	 * @param j            单元格角标，也就是文本集合的角标
	 */
	private void setCellMultiLineText(XWPFTableCell cell, List<String> colList, int j) {
//		cell.setColor("FF0000");
		//　无论一个单元格被切割成多少个run，只需要修改第一个即可，改完就return掉了
		List<XWPFParagraph> paragraphs = cell.getParagraphs();
		for (int i = 0; i< paragraphs.size();i++){
			XWPFParagraph p = paragraphs.get(i);
			List<XWPFRun> runs = p.getRuns();
			//删除旧数据
			for (int z = 0; z< runs.size();z++){
				XWPFRun run = runs.get(z);
				run.removeCarriageReturn();
				p.removeRun(z);
			}
			cell.removeParagraph(i);
		}
		XWPFParagraph p = cell.addParagraph();
		XWPFRun r = p.createRun();
		try {
			String allText = colList.get(j);
			if(StringUtils.equals(allText," \n ")){
				return;
			}
			String[] texts = StringUtils.split(allText, "\n");


			for (int i = 0; i < texts.length; i++) {
				String text = texts[i];
				r.setText(text,i);
				r.addCarriageReturn();
			}
			//染上背景色
//					r.getCTR().addNewRPr().addNewHighlight().setVal(STHighlightColor.YELLOW);
		} catch (Exception e) {
			r.setText("", 0);
		}

	}

	/**
	 * 填充颜色
	 * @param tableIndex    第几个表格，第一个表格为0
	 * @param rowIndex      表格中的第几行，从0开始算起
	 * @param colIndex      行中的第几单元格，从0开始算起
	 * @param color         颜色,rgbStr - the desired cell color, in the hex form "RRGGBB".
	 * @return
	 */
	public WordOperator fillCollorTable(int tableIndex, int rowIndex, int colIndex, String color) {
		// 根据角标取出表格对像
		XWPFTable table = doc.getTables().get(tableIndex);
		if(table == null){
			logger.error("没有找到表格!tableIndex: [{}] row: [{}] col: [{}]",tableIndex,rowIndex,colIndex);
			return this;
		}
		XWPFTableRow row = table.getRow(rowIndex);
		if(row == null){
			logger.error("没有找到row!tableIndex: [{}] row: [{}] col: [{}]",tableIndex,rowIndex,colIndex);
			return this;
		}
		XWPFTableCell col = row.getCell(colIndex);
		if(col == null){
			logger.error("没有找到col!tableIndex: [{}] row: [{}] col: [{}]",tableIndex,rowIndex,colIndex);
			return this;
		}
		col.setColor(color);
		return this;
	}

	/**
	 * 填充颜色
	 * @param tableIndex    第几个表格，第一个表格为0
	 * @param rowIndex      表格中的第几行，从0开始算起
	 * @param color         颜色,rgbStr - the desired cell color, in the hex form "RRGGBB".
	 * @return
	 */
	public WordOperator fillRowCollorTable(int tableIndex, int rowIndex, String color) {
		// 根据角标取出表格对像
		XWPFTable table = doc.getTables().get(tableIndex);
		if(table == null){
			logger.error("没有找到表格!tableIndex: [{}] row: [{}]",tableIndex,rowIndex);
			return this;
		}
		XWPFTableRow row = table.getRow(rowIndex);
		if(row == null){
			logger.error("没有找到row!tableIndex: [{}] row: [{}]",tableIndex,rowIndex);
			return this;
		}
		List<XWPFTableCell> cellList = row.getTableCells();
		cellList.forEach(col -> col.setColor(color));
		return this;
	}

	/**
	 * 填充颜色
	 * @param tableIndex    第几个表格，第一个表格为0
	 * @param colIndex      表格中的第几列，从0开始算起
	 * @param color         颜色,rgbStr - the desired cell color, in the hex form "RRGGBB".
	 * @return
	 */
	public WordOperator fillColCollorTable(int tableIndex, int colIndex, String color) {
		// 根据角标取出表格对像
		XWPFTable table = doc.getTables().get(tableIndex);
		if(table == null){
			logger.error("没有找到表格!tableIndex: [{}] row: [{}]",tableIndex,colIndex);
			return this;
		}
		List<XWPFTableRow> rows = table.getRows();
		rows.forEach(row -> {
			List<XWPFTableCell> cellList = row.getTableCells();
			for (int i = 0; i < cellList.size(); i++) {
				if(i == colIndex){
					cellList.get(i).setColor(color);
				}
			}
		});
		return this;
	}

	/**
	 * 给定输出流即可将word文档导出到
	 */
	public void write(OutputStream os) throws Exception {
		doc.write(os);
	}
	
	/**
	 * 
	 * 功能说明	: 补充空列
	 * 创建者		: byx
	 * 修改日期	: 2018年12月25日
	 * @param returnList:填充到这个list中
	 * @param inputList:输入的list，业务信息列表，用来计算空列的初始值
	 * @param endRowNum:总共N行
	 * @param colNum:拥有几列
	 */
	public static void fillNullCall(List<List<String>> returnList, List<?> inputList, int endRowNum, int colNum){
		int fillNum = 0;
		if(inputList != null && !inputList.isEmpty()){
			fillNum = inputList.size();
		}
		WordOperator.fillNullCall(returnList,fillNum,endRowNum,colNum);

	}
	
	/**
	 * 
	 * 功能说明	: 补充空列
	 * 创建者		: byx
	 * 修改日期	: 2018年12月25日
	 * @param returnList:填充到这个list中
	 * @param startRowNum:从N开始循环
	 * @param endRowNum:总共N行
	 * @param colNum:拥有几列
	 */
	public static void fillNullCall(List<List<String>> returnList, int startRowNum, int endRowNum, int colNum){
		for(int i = startRowNum;i < endRowNum; i++){
			List<String> twoList = Lists.newArrayList();
			for (int j = 0; j < colNum; j++) {
				twoList.add("");
			}
			returnList.add(twoList);
		}
	}

	/**
	 * 根据第几个表格删除第几行，单独删除
	 * @param tableIndex  第几个表格
	 * @param rowIndex  删除第几行
	 * @return
	 */
	public WordOperator removeTableRow(int tableIndex, int rowIndex,int colIndex) {
		// 根据角标取出表格对像
		XWPFTable table = doc.getTables().get(tableIndex);
		if(table == null){
			logger.error("没有找到表格!tableIndex: [{}] row: [{}]",tableIndex,rowIndex);
			return this;
		}
//		XWPFTableRow row = table.getRow(rowIndex);
//		XWPFTableRow nextRow = table.getRow(rowIndex + 1);
//		XWPFTableCell col = row.getCell(colIndex);
//		XWPFTableCell nextCol = nextRow.getCell(colIndex);
//
//		System.out.println("文字内容:" + col.getText());
//		List<String> list = Lists.newArrayList();
//		list.add(StringUtils.isBlank(col.getText())?"空的？？":col.getText());
//		setCellText(nextCol,list,0);
//		XWPFTableCell nextCol = nextRow.getCell(0);
//		if (nextCol.getCTTc().getTcPr() == null) nextCol.getCTTc().addNewTcPr();
//		if (nextCol.getCTTc().getTcPr().getGridSpan() == null) nextCol.getCTTc().getTcPr().addNewGridSpan();
//		nextCol.getCTTc().getTcPr().getGridSpan().setVal(rowSpan.subtract(new BigInteger("1")));
		table.removeRow(rowIndex);
		return this;
	}

	/**
	 * 根据第几个表格删除多行
	 * @param tableIndex 第几个表格
	 * @param rowIndex 删除第几行的集合
	 * @return
	 */
	public WordOperator removeTableRow(int tableIndex, List<Integer> rowIndex,int colIndex) {
		rowIndex.forEach(x -> removeTableRow(tableIndex,x,colIndex));
		return this;
	}

	/**
	 *
	 * 功能说明    : 文档再次替换时使用，因为之前已经替换一次了，所有需要将文档片段存入内存流中，在读取到文档类里，才能替换成功
	 * 创建者        : byx
	 * 修改日期    : 2018年9月26日
	 * @param wo	文档片段对象（已经{替换内容}替换过）
	 * @param result	替换的内容
	 * @return	重新替换的文档片段
	 */
	public static WordOperator twoReplaceWord(WordOperator wo,Map<String, Object> result){
		WordOperator w2o = wo;
		try {
			w2o = twoReplaceWord(wo);
			w2o.replaceTextPlus(result);
		} catch (Exception e) {
			e.printStackTrace();
		}
		//再次替换
		return w2o;
	}

	/**
	 *
	 * 功能说明    : 文档再次替换时使用，因为之前已经替换一次了，所有需要将文档片段存入内存流中，在读取到文档类里，才能替换成功
	 * 创建者        : byx
	 * 修改日期    : 2018年9月26日
	 * @param wo	文档片段对象（已经{替换内容}替换过）
	 * @return	重新替换的文档片段
	 */
	public static WordOperator twoReplaceWord(WordOperator wo) throws Exception {
		//先创建内存输出流
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		//将文档先输出到
		wo.write(output);
		//根据内存输出流创建内存输入流
		InputStream input = new ByteArrayInputStream(output.toByteArray());
		//用这个输入流创建文档对象
		WordOperator w2o = new WordOperator(input);
		IoUtil.close(output);
		IoUtil.close(input);
		return w2o;
	}

}
